package edu.lawrence.auction.interfaces;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.Authentication;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import edu.lawrence.auction.entities.Auction;
import edu.lawrence.auction.entities.Profile;
import edu.lawrence.auction.entities.Shipping;
import edu.lawrence.auction.entities.User;
import edu.lawrence.auction.interfaces.dtos.Bill;
import edu.lawrence.auction.interfaces.dtos.Offer;
import edu.lawrence.auction.interfaces.dtos.ProfileDTO;
import edu.lawrence.auction.interfaces.dtos.Sale;
import edu.lawrence.auction.interfaces.dtos.ShippingDTO;
import edu.lawrence.auction.interfaces.dtos.UserDTO;
import edu.lawrence.auction.security.AuctionUserDetails;
import edu.lawrence.auction.security.JwtService;
import edu.lawrence.auction.security.WrongUserException;
import edu.lawrence.auction.services.DuplicateException;
import edu.lawrence.auction.services.UserService;


@RestController
@RequestMapping("/users")
@CrossOrigin(origins = "*")
public class UserController {

	private UserService us;
	private JwtService jwtService;
    
    public UserController(UserService us,JwtService jwtService) {
        this.us = us;
        this.jwtService = jwtService;
    }

    @PostMapping("/login")
    public ResponseEntity<String> checkLogin(@RequestBody UserDTO user) {
        User result = us.findByNameAndPassword(user.getName(), user.getPassword());
        if (result == null) {
            return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body("Invalid user name or password");
        }
        String token = jwtService.makeJwt(result.getUserid().toString());
        return ResponseEntity.ok().body(token);
    }
    
    @PostMapping
    public ResponseEntity<String> save(@RequestBody UserDTO user) {
        if (user.getName().isBlank() || user.getPassword().isBlank()) {
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body("Empty user name or password");
        }
        String key;
        try {
        	key = us.save(user);
        } catch(DuplicateException ex){
            return ResponseEntity.status(HttpStatus.CONFLICT).body("User with this name already exists");
        } 
        String token = jwtService.makeJwt(key);
        return ResponseEntity.status(HttpStatus.CREATED).body(token);
    }
    
    @PostMapping("/profile")
    public ResponseEntity<String> saveProfile(Authentication authentication,@RequestBody ProfileDTO profile) {
    	AuctionUserDetails details = (AuctionUserDetails) authentication.getPrincipal();
    	UUID id = UUID.fromString(details.getUsername());
    	try {
    		us.saveProfile(id,profile);
    	} catch(WrongUserException ex) {
    		return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body("Invalid user id");
    	} catch(DuplicateException ex) {
    		return ResponseEntity.status(HttpStatus.CONFLICT).body("Duplicate profile");
    	}
    	return ResponseEntity.status(HttpStatus.CREATED).body("Profile created");
    }
    
    @GetMapping("/profile")
    public ResponseEntity<ProfileDTO> getProfile(Authentication authentication) {
    	AuctionUserDetails details = (AuctionUserDetails) authentication.getPrincipal();
    	UUID id = UUID.fromString(details.getUsername());
    	Profile result = us.findProfile(id);
    	if(result == null) {
    		return ResponseEntity.status(HttpStatus.NOT_FOUND).build();
    	}
    	ProfileDTO response = new ProfileDTO(result);
    	return ResponseEntity.ok().body(response);
    }
    
    @GetMapping("/auctions")
	public ResponseEntity<List<Auction>> getAuctions(Authentication authentication) {
    	AuctionUserDetails details = (AuctionUserDetails) authentication.getPrincipal();
    	UUID id = UUID.fromString(details.getUsername());
    	List<Auction> results = us.findAuctions(id);
		return ResponseEntity.ok().body(results);
	}
   
	@PostMapping("/shipping")
    public ResponseEntity<String> postShipping(Authentication authentication,@RequestBody ShippingDTO shipping) {
		AuctionUserDetails details = (AuctionUserDetails) authentication.getPrincipal();
    	UUID id = UUID.fromString(details.getUsername());
    	try {
    		us.saveShipping(id,shipping);
    	} catch(WrongUserException ex) {
    		return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body("Invalid user id");
    	}
    	return ResponseEntity.status(HttpStatus.CREATED).body("Shipping saved.");
    }
    
    @GetMapping("/shipping")
    public ResponseEntity<List<ShippingDTO>> getShipping(Authentication authentication) {
    	AuctionUserDetails details = (AuctionUserDetails) authentication.getPrincipal();
    	UUID id = UUID.fromString(details.getUsername());
    	List<Shipping> results = us.findShipping(id);
    	List<ShippingDTO> response = new ArrayList<ShippingDTO>();
    	for(Shipping s : results) {
    		response.add(new ShippingDTO(s));
    	}
    	return ResponseEntity.ok().body(response);
    }
    
    @GetMapping("/offers")
	public ResponseEntity<List<Offer>> getOffers(Authentication authentication) {
    	AuctionUserDetails details = (AuctionUserDetails) authentication.getPrincipal();
    	UUID id = UUID.fromString(details.getUsername());
    	List<Offer> results = us.findOffers(id);
		return ResponseEntity.ok().body(results);
	}
    
    @GetMapping("/accepted")
    public ResponseEntity<List<Sale>> getSales(Authentication authentication) {
    	AuctionUserDetails details = (AuctionUserDetails) authentication.getPrincipal();
    	UUID id = UUID.fromString(details.getUsername());
    	List<Sale> results = us.findSales(id);
    	return ResponseEntity.ok().body(results);
    }
    
    @GetMapping("/billed")
    public ResponseEntity<List<Bill>> getBills(Authentication authentication) {
    	AuctionUserDetails details = (AuctionUserDetails) authentication.getPrincipal();
    	UUID id = UUID.fromString(details.getUsername());
    	List<Bill> results = us.findBills(id);
    	return ResponseEntity.ok().body(results);
    }
    
    @GetMapping("/sold")
    public ResponseEntity<List<Bill>> getSold(Authentication authentication) {
    	AuctionUserDetails details = (AuctionUserDetails) authentication.getPrincipal();
    	UUID id = UUID.fromString(details.getUsername());
    	List<Bill> results = us.findSoldBills(id);
    	return ResponseEntity.ok().body(results);
    }
    
    @GetMapping("/shipped")
    public ResponseEntity<List<Bill>> getShipped(Authentication authentication) {
    	AuctionUserDetails details = (AuctionUserDetails) authentication.getPrincipal();
    	UUID id = UUID.fromString(details.getUsername());
    	List<Bill> results = us.findShippedBills(id);
    	return ResponseEntity.ok().body(results);
    }
}